<?php

declare(strict_types=1);

namespace Drupal\Tests\swiffy_slider\Kernel;

use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Tests the swiffy slider formatter functionality.
 *
 * @group swiffy_slider
 */
class SwiffySliderEntityReferenceFormatterTest extends EntityKernelTestBase {

  /**
   * Plugin id of field formatter.
   */
  public const FORMATTER = 'swiffy_slider_entity_reference';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'swiffy_slider',
    'swiffy_slider_content_type',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    \Drupal::service('theme_installer')->install(['stark']);
    $this->config('system.theme')->set('default', 'stark')->save();
    $this->installConfig([
      'system',
      'node',
      'user',
      'swiffy_slider_content_type',
    ]);
    $role = Role::load(RoleInterface::ANONYMOUS_ID);
    if ($role instanceof RoleInterface) {
      $role
        ->grantPermission('access content')
        ->save();
    }
  }

  /**
   * Test html output of field formatter.
   */
  public function testFieldFormatter(): void {
    $item_ref_1 = Node::create(['type' => 'swiffy_slider_ref_item']);
    $item_ref_1->setTitle('Item 1');
    $item_ref_1->save();

    $item_ref_2 = Node::create(['type' => 'swiffy_slider_ref_item']);
    $item_ref_2->setTitle('Item 2');
    $item_ref_2->save();

    $node = Node::create(['type' => 'swiffy_slider_test']);
    $node->setTitle('test');
    $value = [];
    $value[] = ['target_id' => $item_ref_1->id()];
    $value[] = ['target_id' => $item_ref_2->id()];
    $node->set('field_ref_item', $value);

    $node->save();
    $view_builder = $this->entityTypeManager->getViewBuilder('node');
    $build = $view_builder->view($node, 'default');
    $output = $this->container->get('renderer')->renderRoot($build);
    $this->setRawContent((string) $output);
    $this->assertRaw('<div  class="swiffy-slider">');
    $this->assertRaw('<ul style="list-style:none" class="slider-container">');
    $this->assertRaw('<span>Item 1</span>');
    $this->assertRaw('<span>Item 2</span>');
    $this->assertRaw('<button type="button" class="slider-nav" aria-label="Go to previous"></button>');
    $this->assertRaw('<button type="button" class="slider-nav slider-nav-next" aria-label="Go to next"></button>');
    $this->assertRaw('<button class="active"  aria-label="Go to slide"></button>');
    $this->assertRaw('<button  aria-label="Go to slide"></button>');
    $this->assertNoRaw('<ul class="links inline"><li><a href="/node/1" rel="tag" title="Item 1" hreflang="en">Read more<span class="visually-hidden"> about Item 1</span></a></li></ul>');
    $this->assertNoRaw('<ul class="links inline"><li><a href="/node/2" rel="tag" title="Item 2" hreflang="en">Read more<span class="visually-hidden"> about Item 2</span></a></li></ul>');

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $repository */
    $repository = $this->container->get('entity_display.repository');
    $display = $repository->getViewDisplay('node', 'swiffy_slider_test');
    $component = $display->getComponent('field_ref_item');
    $component['settings']['view_mode'] = 'teaser';
    $component['settings']['swiffy_slider_permalink'] = 'https://example.org/configuration/?slider-item-ratio-value=slider-item-ratio-1x1&slider-nav-dark=slider-nav-dark';
    $display->setComponent('field_ref_item', $component)->save();

    $view_builder = $this->entityTypeManager->getViewBuilder('node');
    $build = $view_builder->view($node, 'default');
    $output = $this->container->get('renderer')->renderRoot($build);
    $this->setRawContent((string) $output);
    $this->assertRaw('<div  class="swiffy-slider slider-item-ratio-1x1 slider-nav-dark">');
    $this->assertRaw('<ul class="links inline"><li><a href="/node/1" rel="tag" title="Item 1" hreflang="en">Read more<span class="visually-hidden"> about Item 1</span></a></li></ul>');
    $this->assertRaw('<ul class="links inline"><li><a href="/node/2" rel="tag" title="Item 2" hreflang="en">Read more<span class="visually-hidden"> about Item 2</span></a></li></ul>');
  }

}
